var gutil     = require('gulp-util'),
  through   = require('through2'),
  gzipSize  = require('gzip-size'),
  yaml      = require('yamljs'),
  fs        = require('fs');

module.exports = function (options) {
  "use strict";

  options          = options || {};
  options.gzip     = (options.gzip   !== undefined) ? options.gzip   : false;
  options.minifier = (options.minifier !== undefined) ? options.minifier : null;
  options.total    = (options.total  !== undefined) ? options.total  : true;
  options.path     = (options.path  !== undefined) ? options.path  : 'config/file_sizes.yml';

  var fileCount = 0,
    totalSize = 0,
    gzippedSize;

  var fileMap = {
        components: {},
        elements: {},
        features: {},
        base: {},
        adTag: { tags: {}, channels: {}, utils: {} }
    };

  return through.obj(function (file, enc, callback) {
    if (file.isNull()) {
      callback(null, file);
      return;
    }

    if (file.isStream()) {
      callback(new gutil.PluginError('gulp-size', 'Streaming not supported'));
      return;
    }

    totalSize += file.contents.length;
    fileCount ++;

    gzippedSize = gzipSize.sync(file.contents);

    // Adding minified size to file map
    var fileType     = file.relative.match(/\.js/) ? 'js' : 'css',
      fullPath     = file.relative.replace(/\.js/g, '').replace(/\.css/g, ''),
      parsedFilePath = fullPath.split('/'),
      filename     = parsedFilePath[parsedFilePath.length - 1];

    if (fileType == 'js') {

      if (parsedFilePath.length === 1) {
        // vendor / base files
        if (filename === 'video') {
          fileMap['components']['video'] = (fileMap['components']['video'] || 0) + gzippedSize;
        }
        else if (filename === 'swipe') {
          fileMap['components']['mediaCarousel'] = (fileMap['components']['mediaCarousel'] || 0) + gzippedSize;
        }
        else if (filename === 'zepto' || filename.match(/promise/i)) {
          fileMap['base']['ad'] = (fileMap['base']['ad'] || 0) + gzippedSize;
        }
        else if (filename === 'mustache') {
          fileMap['elements']['customElement'] = (fileMap['elements']['customElement'] || 0) + gzippedSize;
        }
        else {
          fileMap['base'][filename] = (fileMap['base'][filename] || 0) + gzippedSize;
        }
      }
      else if (parsedFilePath.length === 2 && parsedFilePath[0] === 'html') {
        // base files in ads/html directory
        if (parsedFilePath[1] === 'mediaCarouselNavigation' || parsedFilePath[1] === 'slide') {
          fileMap['components']['mediaCarousel'] = (fileMap['components']['mediaCarousel'] || 0) + gzippedSize;
        } else {
          fileMap['base'][filename] = (fileMap['base'][filename] || 0) + gzippedSize;
        }
      }
      else if (fullPath.match(/components/)) {
        if (filename.match(/video/)) {
          fileMap['components']['video'] = (fileMap['components']['video'] || 0) + gzippedSize;
        }
        else {
          fileMap['components'][filename] = (fileMap['components'][filename] || 0) + gzippedSize;
        }
      }
      else if (fullPath.match(/features/)) {
        fileMap['features'][filename] = (fileMap['features'][filename] || 0) + gzippedSize;
      }
      else if (fullPath.match(/elements/)) {
        if (filename === 'thumbnail') {
          fileMap['components']['mediaCarousel'] = (fileMap['components']['mediaCarousel'] || 0) + gzippedSize;
        }
        else {
          fileMap['elements'][filename] = (fileMap['elements'][filename] || 0) + gzippedSize;
        }
      }
    }
    else if (fileType == 'css') {
      if (parsedFilePath[0] === 'platform') {
        fileMap['base']['ad'] = (fileMap['base']['ad'] || 0) + gzippedSize;
      }
      else if (fullPath.match('video')) {
        fileMap['components']['video'] = (fileMap['components']['video'] || 0) + gzippedSize;
      }
      else if (fullPath.match('components')) {
        fileMap['components'][filename] = (fileMap['components'][filename] || 0) + gzippedSize;
      }
    }

  callback(null, file);

  }, function (callback) {

    if (fileCount > 0) {
      var yamlString = yaml.stringify(fileMap);
      fs.writeFile(options.path, yamlString, function(err) {
        if (err) {
          console.log('error writing to file: ', err);
        }
        else {
          console.log('file written to: ', options.path);
        }
      })
    }

    return callback();
  });
};
